@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwidget.h>
@@ -37,7 +38,6 @@
 #include <gtk/gtkfilesel.h>
 #include <gtk/gtksignal.h>
 
-#include "intl.h"
 #include "main.h"
 #include "inc.h"
 #include "mbox.h"
@@ -63,7 +63,7 @@ static void export_cancel_cb(GtkWidget *widget, gpointer data);
 static void export_srcsel_cb(GtkWidget *widget, gpointer data);
 static void export_filesel_cb(GtkWidget *widget, gpointer data);
 static gint delete_event(GtkWidget *widget, GdkEventAny *event, gpointer data);
-static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data);
+static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data);
 
 gint export_mbox(FolderItem *default_src)
 {
@@ -93,18 +93,29 @@ gint export_mbox(FolderItem *default_src)
 	gtk_main();
 
 	if (export_ack) {
-		gchar *srcdir, *mbox;
+		const gchar *srcdir, *utf8mbox;
 		FolderItem *src;
 
 		srcdir = gtk_entry_get_text(GTK_ENTRY(src_entry));
-		mbox = gtk_entry_get_text(GTK_ENTRY(file_entry));
+		utf8mbox = gtk_entry_get_text(GTK_ENTRY(file_entry));
+
+		if (utf8mbox && *utf8mbox) {
+			gchar *mbox;
+
+			mbox = g_filename_from_utf8
+				(utf8mbox, -1, NULL, NULL, NULL);
+			if (!mbox) {
+				g_warning("faild to convert character set\n");
+				mbox = g_strdup(utf8mbox);
+			}
 
-		if (mbox && *mbox) {
 			src = folder_find_item_from_identifier(srcdir);
 			if (!src)
 				g_warning("Can't find the folder.\n");
 			else
 				ok = export_to_mbox(src, mbox);
+
+			g_free(mbox);
 		}
 	}
 
@@ -123,16 +134,16 @@ static void export_create(void)
 	GtkWidget *src_label;
 	GtkWidget *confirm_area;
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(window), _("Export"));
 	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(delete_event), NULL);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(key_pressed), NULL);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(delete_event), NULL);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
 
 	vbox = gtk_vbox_new(FALSE, 4);
@@ -151,7 +162,7 @@ static void export_create(void)
 	gtk_container_set_border_width(GTK_CONTAINER(table), 8);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 8);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
-	gtk_widget_set_usize(table, 420, -1);
+	gtk_widget_set_size_request(table, 420, -1);
 
 	src_label = gtk_label_new(_("Source dir:"));
 	gtk_table_attach(GTK_TABLE(table), src_label, 0, 1, 0, 1,
@@ -174,26 +185,26 @@ static void export_create(void)
 	src_button = gtk_button_new_with_label(_(" Select... "));
 	gtk_table_attach(GTK_TABLE(table), src_button, 2, 3, 0, 1,
 			 0, 0, 0, 0);
-	gtk_signal_connect(GTK_OBJECT(src_button), "clicked",
-			   GTK_SIGNAL_FUNC(export_srcsel_cb), NULL);
+	g_signal_connect(G_OBJECT(src_button), "clicked",
+			 G_CALLBACK(export_srcsel_cb), NULL);
 
 	file_button = gtk_button_new_with_label(_(" Select... "));
 	gtk_table_attach(GTK_TABLE(table), file_button, 2, 3, 1, 2,
 			 0, 0, 0, 0);
-	gtk_signal_connect(GTK_OBJECT(file_button), "clicked",
-			   GTK_SIGNAL_FUNC(export_filesel_cb), NULL);
+	g_signal_connect(G_OBJECT(file_button), "clicked",
+			 G_CALLBACK(export_filesel_cb), NULL);
 
-	gtkut_button_set_create(&confirm_area,
-				&ok_button,	_("OK"),
-				&cancel_button, _("Cancel"),
-				NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area,
+				      &ok_button, GTK_STOCK_OK,
+				      &cancel_button, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_button);
 
-	gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
-			   GTK_SIGNAL_FUNC(export_ok_cb), NULL);
-	gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
-			   GTK_SIGNAL_FUNC(export_cancel_cb), NULL);
+	g_signal_connect(G_OBJECT(ok_button), "clicked",
+			 G_CALLBACK(export_ok_cb), NULL);
+	g_signal_connect(G_OBJECT(cancel_button), "clicked",
+			 G_CALLBACK(export_cancel_cb), NULL);
 
 	gtk_widget_show_all(window);
 }
@@ -215,10 +226,21 @@ static void export_cancel_cb(GtkWidget *widget, gpointer data)
 static void export_filesel_cb(GtkWidget *widget, gpointer data)
 {
 	gchar *filename;
+	gchar *utf8_filename;
 
-	filename = filesel_select_file(_("Select exporting file"), NULL);
-	if (filename)
-		gtk_entry_set_text(GTK_ENTRY(file_entry), filename);
+	filename = filesel_select_file(_("Select exporting file"), NULL,
+				       GTK_FILE_CHOOSER_ACTION_SAVE);
+	if (!filename) return;
+
+	utf8_filename = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
+	if (!utf8_filename) {
+		g_warning("export_filesel_cb(): faild to convert character set.\n");
+		utf8_filename = g_strdup(filename);
+	}
+	gtk_entry_set_text(GTK_ENTRY(file_entry), utf8_filename);
+	g_free(utf8_filename);
+
+	g_free(filename);
 }
 
 static void export_srcsel_cb(GtkWidget *widget, gpointer data)
@@ -236,8 +258,9 @@ static gint delete_event(GtkWidget *widget, GdkEventAny *event, gpointer data)
 	return TRUE;
 }
 
-static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
+static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
 		export_cancel_cb(NULL, NULL);
+	return FALSE;
 }
