@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <dirent.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -36,12 +37,12 @@
 #  include <sys/time.h>
 #endif
 
-#include "intl.h"
 #include "folder.h"
 #include "mh.h"
 #include "procmsg.h"
 #include "procheader.h"
 #include "utils.h"
+#include "prefs_common.h"
 
 static void	mh_folder_init		(Folder		*folder,
 					 const gchar	*name,
@@ -107,6 +108,9 @@ static FolderItem *mh_create_folder	(Folder		*folder,
 static gint    mh_rename_folder		(Folder		*folder,
 					 FolderItem	*item,
 					 const gchar	*name);
+static gint    mh_move_folder		(Folder		*folder,
+					 FolderItem	*item,
+					 FolderItem	*new_parent);
 static gint    mh_remove_folder		(Folder		*folder,
 					 FolderItem	*item);
 
@@ -155,6 +159,7 @@ static FolderClass mh_class =
 
 	mh_create_folder,
 	mh_rename_folder,
+	mh_move_folder,
 	mh_remove_folder,
 };
 
@@ -204,21 +209,46 @@ static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
 	if (use_cache && item->mtime == cur_mtime) {
 		debug_print("Folder is not modified.\n");
 		mlist = procmsg_read_cache(item, FALSE);
-		if (!mlist)
+		if (!mlist) {
 			mlist = mh_get_uncached_msgs(NULL, item);
+			if (mlist)
+				item->cache_dirty = TRUE;
+		}
 	} else if (use_cache) {
-		GSList *newlist;
+		GSList *newlist, *cur, *next;
+		gboolean strict_cache_check = prefs_common.strict_cache_check;
 
-		mlist = procmsg_read_cache(item, TRUE);
+		if (item->stype == F_QUEUE || item->stype == F_DRAFT)
+			strict_cache_check = TRUE;
+
+		mlist = procmsg_read_cache(item, strict_cache_check);
 		msg_table = procmsg_msg_hash_table_create(mlist);
-
 		newlist = mh_get_uncached_msgs(msg_table, item);
+		if (newlist)
+			item->cache_dirty = TRUE;
 		if (msg_table)
 			g_hash_table_destroy(msg_table);
 
+		if (!strict_cache_check) {
+			/* remove nonexistent messages */
+			for (cur = mlist; cur != NULL; cur = next) {
+				MsgInfo *msginfo = (MsgInfo *)cur->data;
+				next = cur->next;
+				if (!MSG_IS_CACHED(msginfo->flags)) {
+					debug_print("removing nonexistent message %d from cache\n", msginfo->msgnum);
+					mlist = g_slist_remove(mlist, msginfo);
+					procmsg_msginfo_free(msginfo);
+					item->cache_dirty = TRUE;
+					item->mark_dirty = TRUE;
+				}
+			}
+		}
+
 		mlist = g_slist_concat(mlist, newlist);
-	} else
+	} else {
 		mlist = mh_get_uncached_msgs(NULL, item);
+		item->cache_dirty = TRUE;
+	}
 
 	item->mtime = cur_mtime;
 
@@ -233,6 +263,8 @@ static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
 	g_print("mh_get_msg_list: %s: elapsed time: %ld.%06ld sec\n",
 		item->path, tv_result.tv_sec, tv_result.tv_usec);
 #endif
+	debug_print("cache_dirty: %d, mark_dirty: %d\n",
+		    item->cache_dirty, item->mark_dirty);
 
 	return mlist;
 }
@@ -641,7 +673,7 @@ static gint mh_remove_msg(Folder *folder, FolderItem *item, MsgInfo *msginfo)
 	MSG_SET_TMP_FLAGS(msginfo->flags, MSG_INVALID);
 
 	if (msginfo->msgnum == item->last_num)
-		item->last_num = mh_scan_folder_full(folder, item, FALSE);
+		mh_scan_folder_full(folder, item, FALSE);
 
 	return 0;
 }
@@ -823,6 +855,7 @@ static FolderItem *mh_create_folder(Folder *folder, FolderItem *parent,
 				    const gchar *name)
 {
 	gchar *path;
+	gchar *fs_name;
 	gchar *fullpath;
 	FolderItem *new_item;
 
@@ -831,7 +864,10 @@ static FolderItem *mh_create_folder(Folder *folder, FolderItem *parent,
 	g_return_val_if_fail(name != NULL, NULL);
 
 	path = folder_item_get_path(parent);
-	fullpath = g_strconcat(path, G_DIR_SEPARATOR_S, name, NULL);
+	fs_name = g_filename_from_utf8(name, -1, NULL, NULL, NULL);
+	fullpath = g_strconcat(path, G_DIR_SEPARATOR_S,
+			       fs_name ? fs_name : name, NULL);
+	g_free(fs_name);
 	g_free(path);
 
 	if (make_dir(fullpath) < 0) {
@@ -853,43 +889,107 @@ static FolderItem *mh_create_folder(Folder *folder, FolderItem *parent,
 	return new_item;
 }
 
-static gint mh_rename_folder(Folder *folder, FolderItem *item,
-			     const gchar *name)
+static gint mh_move_folder_real(Folder *folder, FolderItem *item,
+				FolderItem *new_parent, const gchar *name)
 {
 	gchar *oldpath;
-	gchar *dirname;
 	gchar *newpath;
+	gchar *dirname;
+	gchar *new_dir;
+	gchar *name_;
+	gchar *utf8_name;
 	gchar *paths[2];
 
 	g_return_val_if_fail(folder != NULL, -1);
 	g_return_val_if_fail(item != NULL, -1);
+	g_return_val_if_fail(folder == item->folder, -1);
 	g_return_val_if_fail(item->path != NULL, -1);
-	g_return_val_if_fail(name != NULL, -1);
+	g_return_val_if_fail(new_parent != NULL || name != NULL, -1);
+	if (new_parent) {
+		g_return_val_if_fail(item != new_parent, -1);
+		g_return_val_if_fail(item->parent != new_parent, -1);
+		g_return_val_if_fail(item->folder == new_parent->folder, -1);
+		if (g_node_is_ancestor(item->node, new_parent->node)) {
+			g_warning("folder to be moved is ancestor of new parent\n");
+			return -1;
+		}
+	}
 
 	oldpath = folder_item_get_path(item);
-	dirname = g_dirname(oldpath);
-	newpath = g_strconcat(dirname, G_DIR_SEPARATOR_S, name, NULL);
-	g_free(dirname);
+	if (new_parent) {
+		if (name) {
+			name_ = g_filename_from_utf8(name, -1, NULL, NULL,
+						     NULL);
+			if (!name_)
+				name_ = g_strdup(name);
+			utf8_name = g_strdup(name);
+		} else {
+			name_ = g_path_get_basename(oldpath);
+			utf8_name = g_filename_to_utf8(name_, -1, NULL, NULL,
+						       NULL);
+			if (!utf8_name)
+				utf8_name = g_strdup(name_);
+		}
+		new_dir = folder_item_get_path(new_parent);
+		newpath = g_strconcat(new_dir, G_DIR_SEPARATOR_S, name_, NULL);
+		g_free(new_dir);
+	} else {
+		name_ = g_filename_from_utf8(name, -1, NULL, NULL, NULL);
+		utf8_name = g_strdup(name);
+		dirname = g_dirname(oldpath);
+		newpath = g_strconcat(dirname, G_DIR_SEPARATOR_S,
+				      name_ ? name_ : name, NULL);
+		g_free(dirname);
+	}
+	g_free(name_);
+
+	if (is_file_entry_exist(newpath)) {
+		g_warning("%s already exists\n", newpath);
+		g_free(oldpath);
+		g_free(newpath);
+		g_free(utf8_name);
+		return -1;
+	}
+
+	debug_print("mh_move_folder: rename(%s, %s)\n", oldpath, newpath);
 
 	if (rename(oldpath, newpath) < 0) {
 		FILE_OP_ERROR(oldpath, "rename");
 		g_free(oldpath);
 		g_free(newpath);
+		g_free(utf8_name);
 		return -1;
 	}
 
 	g_free(oldpath);
 	g_free(newpath);
 
-	if (strchr(item->path, G_DIR_SEPARATOR) != NULL) {
-		dirname = g_dirname(item->path);
-		newpath = g_strconcat(dirname, G_DIR_SEPARATOR_S, name, NULL);
-		g_free(dirname);
-	} else
-		newpath = g_strdup(name);
+	if (new_parent) {
+		g_node_unlink(item->node);
+		g_node_append(new_parent->node, item->node);
+		item->parent = new_parent;
+		if (new_parent->path != NULL) {
+			newpath = g_strconcat(new_parent->path,
+					      G_DIR_SEPARATOR_S, utf8_name,
+					      NULL);
+			g_free(utf8_name);
+		} else
+			newpath = utf8_name;
+	} else {
+		if (strchr(item->path, G_DIR_SEPARATOR) != NULL) {
+			dirname = g_dirname(item->path);
+			newpath = g_strconcat(dirname, G_DIR_SEPARATOR_S,
+					      utf8_name, NULL);
+			g_free(dirname);
+			g_free(utf8_name);
+		} else
+			newpath = utf8_name;
+	}
 
-	g_free(item->name);
-	item->name = g_strdup(name);
+	if (name) {
+		g_free(item->name);
+		item->name = g_strdup(name);
+	}
 
 	paths[0] = g_strdup(item->path);
 	paths[1] = newpath;
@@ -898,9 +998,22 @@ static gint mh_rename_folder(Folder *folder, FolderItem *item,
 
 	g_free(paths[0]);
 	g_free(paths[1]);
+
 	return 0;
 }
 
+static gint mh_move_folder(Folder *folder, FolderItem *item,
+			   FolderItem *new_parent)
+{
+	return mh_move_folder_real(folder, item, new_parent, NULL);
+}
+
+static gint mh_rename_folder(Folder *folder, FolderItem *item,
+			     const gchar *name)
+{
+	return mh_move_folder_real(folder, item, NULL, name);
+}
+
 static gint mh_remove_folder(Folder *folder, FolderItem *item)
 {
 	gchar *path;
@@ -971,7 +1084,9 @@ static GSList *mh_get_uncached_msgs(GHashTable *msg_table, FolderItem *item)
 			msginfo = g_hash_table_lookup
 				(msg_table, GUINT_TO_POINTER(num));
 
-			if (!msginfo) {
+			if (msginfo) {
+				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_CACHED);
+			} else {
 				/* not found in the cache (uncached message) */
 				msginfo = mh_parse_msg(d->d_name, item);
 				if (!msginfo) continue;
@@ -1112,7 +1227,10 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	DIR *dp;
 	struct dirent *d;
 	struct stat s;
+	gchar *fs_path;
 	gchar *entry;
+	gchar *utf8entry;
+	gchar *utf8name;
 	gint n_msg = 0;
 
 	g_return_if_fail(item != NULL);
@@ -1120,11 +1238,18 @@ static void mh_scan_tree_recursive(FolderItem *item)
 
 	folder = item->folder;
 
-	dp = opendir(item->path ? item->path : ".");
+	fs_path = item->path ?
+		g_filename_from_utf8(item->path, -1, NULL, NULL, NULL)
+		: g_strdup(".");
+	if (!fs_path)
+		fs_path = g_strdup(item->path);
+	dp = opendir(fs_path);
 	if (!dp) {
-		FILE_OP_ERROR(item->path ? item->path : ".", "opendir");
+		FILE_OP_ERROR(fs_path, "opendir");
+		g_free(fs_path);
 		return;
 	}
+	g_free(fs_path);
 
 	debug_print("scanning %s ...\n",
 		    item->path ? item->path
@@ -1135,11 +1260,18 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	while ((d = readdir(dp)) != NULL) {
 		if (d->d_name[0] == '.') continue;
 
+		utf8name = g_filename_to_utf8(d->d_name, -1, NULL, NULL, NULL);
+		if (!utf8name)
+			utf8name = g_strdup(d->d_name);
+
 		if (item->path)
-			entry = g_strconcat(item->path, G_DIR_SEPARATOR_S,
-					    d->d_name, NULL);
+			utf8entry = g_strconcat(item->path, G_DIR_SEPARATOR_S,
+						utf8name, NULL);
 		else
-			entry = g_strdup(d->d_name);
+			utf8entry = g_strdup(utf8name);
+		entry = g_filename_from_utf8(utf8entry, -1, NULL, NULL, NULL);
+		if (!entry)
+			entry = g_strdup(utf8entry);
 
 		if (
 #ifdef HAVE_DIRENT_D_TYPE
@@ -1157,21 +1289,37 @@ static void mh_scan_tree_recursive(FolderItem *item)
 #if 0
 			if (mh_is_maildir(entry)) {
 				g_free(entry);
+				g_free(utf8entry);
+				g_free(utf8name);
 				continue;
 			}
 #endif
+			if (g_utf8_validate(utf8name, -1, NULL) == FALSE) {
+				g_warning(_("Directory name\n"
+					    "'%s' is not a valid UTF-8 string.\n"
+					    "Maybe the locale encoding is used for filename.\n"
+					    "If that is the case, you must set the following environmental variable\n"
+					    "(see README for detail):\n"
+					    "\n"
+					    "\tG_FILENAME_ENCODING=@locale\n"),
+					  utf8name);
+				g_free(entry);
+				g_free(utf8entry);
+				g_free(utf8name);
+				continue;
+			}
 
 			node = item->node;
 			for (node = node->children; node != NULL; node = node->next) {
 				FolderItem *cur_item = FOLDER_ITEM(node->data);
-				if (!strcmp2(cur_item->path, entry)) {
+				if (!strcmp2(cur_item->path, utf8entry)) {
 					new_item = cur_item;
 					break;
 				}
 			}
 			if (!new_item) {
 				debug_print("new folder '%s' found.\n", entry);
-				new_item = folder_item_new(d->d_name, entry);
+				new_item = folder_item_new(utf8name, utf8entry);
 				folder_item_append(item, new_item);
 			}
 
@@ -1203,6 +1351,8 @@ static void mh_scan_tree_recursive(FolderItem *item)
 		} else if (to_number(d->d_name) > 0) n_msg++;
 
 		g_free(entry);
+		g_free(utf8entry);
+		g_free(utf8name);
 	}
 
 	closedir(dp);
