@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
@@ -31,7 +32,6 @@
 #include <string.h>
 #include <errno.h>
 
-#include "intl.h"
 #include "main.h"
 #include "prefs.h"
 #include "prefs_filter.h"
@@ -234,7 +234,7 @@ static FilterRule *prefs_filter_edit_dialog_to_rule	(void);
 static gint prefs_filter_edit_deleted		(GtkWidget	*widget,
 						 GdkEventAny	*event,
 						 gpointer	 data);
-static gint prefs_filter_edit_key_pressed	(GtkWidget	*widget,
+static gboolean prefs_filter_edit_key_pressed	(GtkWidget	*widget,
 						 GdkEventKey	*event,
 						 gpointer	 data);
 static void prefs_filter_edit_ok		(void);
@@ -260,8 +260,14 @@ static void prefs_filter_action_add_cb		(GtkWidget	*widget,
 
 FilterRule *prefs_filter_edit_open(FilterRule *rule, const gchar *header)
 {
+	static gboolean lock = FALSE;
 	FilterRule *new_rule;
 
+	if (lock)
+		return NULL;
+
+	lock = TRUE;
+
 	if (!rule_edit_window.window)
 		prefs_filter_edit_create();
 
@@ -271,6 +277,7 @@ FilterRule *prefs_filter_edit_open(FilterRule *rule, const gchar *header)
 	prefs_filter_edit_rule_to_dialog(rule);
 	if (header)
 		prefs_filter_edit_activate_cond_header(header);
+	GTK_EVENTS_FLUSH();
 	gtk_widget_show(rule_edit_window.window);
 
 	rule_edit_window.new_rule = NULL;
@@ -278,9 +285,9 @@ FilterRule *prefs_filter_edit_open(FilterRule *rule, const gchar *header)
 	while (rule_edit_window.edit_finished == FALSE)
 		gtk_main_iteration();
 
+	gtk_widget_hide(rule_edit_window.window);
 	prefs_filter_edit_clear();
 	prefs_filter_set_msg_header_list(NULL);
-	gtk_widget_hide(rule_edit_window.window);
 
 	new_rule = rule_edit_window.new_rule;
 	rule_edit_window.new_rule = NULL;
@@ -288,6 +295,8 @@ FilterRule *prefs_filter_edit_open(FilterRule *rule, const gchar *header)
 	if (new_rule)
 		debug_print("new rule created: %s\n", new_rule->name);
 
+	lock = FALSE;
+
 	return new_rule;
 }
 
@@ -315,9 +324,9 @@ static void prefs_filter_edit_create(void)
 	GtkWidget *cancel_btn;
 	GtkWidget *confirm_area;
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width(GTK_CONTAINER(window), 8);
-	gtk_widget_set_usize(window, 632, 405);
+	gtk_widget_set_size_request(window, 632, 405);
 	gtk_window_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
@@ -327,24 +336,24 @@ static void prefs_filter_edit_create(void)
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
-	gtkut_button_set_create(&confirm_area, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
 	gtk_window_set_title(GTK_WINDOW(window),
 			     _("Filter rule"));
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(prefs_filter_edit_deleted), NULL);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(prefs_filter_edit_key_pressed),
-			   NULL);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(prefs_filter_edit_deleted), NULL);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(prefs_filter_edit_key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT (window);
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_edit_ok), NULL);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_edit_cancel), NULL);
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(prefs_filter_edit_ok), NULL);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(prefs_filter_edit_cancel), NULL);
 
 	hbox = gtk_hbox_new(FALSE, 4);
 	gtk_widget_show(hbox);
@@ -367,6 +376,7 @@ static void prefs_filter_edit_create(void)
 	gtk_box_pack_start(GTK_BOX(hbox), bool_op_optmenu, FALSE, FALSE, 0);
 
 	menu = gtk_menu_new();
+	gtk_widget_show(menu);
 	MENUITEM_ADD(menu, menuitem,
 		     _("If any of the following condition matches"), FLT_OR);
 	MENUITEM_ADD(menu, menuitem,
@@ -375,7 +385,7 @@ static void prefs_filter_edit_create(void)
 
 	cond_scrolled_win = gtk_scrolled_window_new(NULL, NULL);
 	gtk_widget_show(cond_scrolled_win);
-	gtk_widget_set_usize(cond_scrolled_win, -1, 125);
+	gtk_widget_set_size_request(cond_scrolled_win, -1, 125);
 	gtk_box_pack_start(GTK_BOX(vbox), cond_scrolled_win, TRUE, TRUE, 0);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(cond_scrolled_win),
 				       GTK_POLICY_AUTOMATIC,
@@ -468,6 +478,11 @@ static void prefs_filter_edit_rule_to_dialog(FilterRule *rule)
 			 index);
 	}
 
+	gtkut_scrolled_window_reset_position
+		(GTK_SCROLLED_WINDOW(rule_edit_window.cond_scrolled_win));
+	gtkut_scrolled_window_reset_position
+		(GTK_SCROLLED_WINDOW(rule_edit_window.action_scrolled_win));
+
 	prefs_filter_edit_add_rule_cond(rule);
 	prefs_filter_edit_add_rule_action(rule);
 }
@@ -537,8 +552,8 @@ static CondHBox *prefs_filter_edit_cond_hbox_create(void)
 	GtkWidget *label;
 	GtkWidget *del_btn;
 	GtkWidget *add_btn;
-	GtkWidget *del_pixmap;
-	GtkWidget *add_pixmap;
+	GtkWidget *del_img;
+	GtkWidget *add_img;
 
 	cond_hbox = g_new0(CondHBox, 1);
 
@@ -552,12 +567,13 @@ static CondHBox *prefs_filter_edit_cond_hbox_create(void)
 #define COND_MENUITEM_ADD(str, action)					    \
 {									    \
 	MENUITEM_ADD(menu, menuitem, str, action);			    \
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",		    \
-			   GTK_SIGNAL_FUNC(prefs_filter_cond_activated_cb), \
-			   cond_hbox);				    \
+	g_signal_connect(G_OBJECT(menuitem), "activate",		    \
+			 G_CALLBACK(prefs_filter_cond_activated_cb),	    \
+			 cond_hbox);					    \
 }
 
 	menu = gtk_menu_new();
+	gtk_widget_show(menu);
 	MENUITEM_ADD(menu, menuitem, NULL,        PF_COND_SEPARATOR);
 	COND_MENUITEM_ADD(_("To or Cc"),          PF_COND_TO_OR_CC);
 	COND_MENUITEM_ADD(_("Any header"),        PF_COND_ANY_HEADER);
@@ -579,6 +595,7 @@ static CondHBox *prefs_filter_edit_cond_hbox_create(void)
 	gtk_box_pack_start(GTK_BOX(hbox), match_type_optmenu, FALSE, FALSE, 0);
 
 	menu = gtk_menu_new();
+	gtk_widget_show(menu);
 	MENUITEM_ADD(menu, menuitem, _("contains"),
 		     PF_MATCH_CONTAIN);
 	MENUITEM_ADD(menu, menuitem, _("doesn't contain"),
@@ -594,17 +611,21 @@ static CondHBox *prefs_filter_edit_cond_hbox_create(void)
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(match_type_optmenu), menu);
 
 	size_match_optmenu = gtk_option_menu_new();
+	gtk_widget_show(size_match_optmenu);
 	gtk_box_pack_start(GTK_BOX(hbox), size_match_optmenu, FALSE, FALSE, 0);
 
 	menu = gtk_menu_new();
+	gtk_widget_show(menu);
 	MENUITEM_ADD(menu, menuitem, _("is larger than"), PF_SIZE_LARGER);
 	MENUITEM_ADD(menu, menuitem, _("is smaller than"), PF_SIZE_SMALLER);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(size_match_optmenu), menu);
 
 	age_match_optmenu = gtk_option_menu_new();
+	gtk_widget_show(age_match_optmenu);
 	gtk_box_pack_start(GTK_BOX(hbox), age_match_optmenu, FALSE, FALSE, 0);
 
 	menu = gtk_menu_new();
+	gtk_widget_show(menu);
 	MENUITEM_ADD(menu, menuitem, _("is longer than"), PF_AGE_LONGER);
 	MENUITEM_ADD(menu, menuitem, _("is shorter than"), PF_AGE_SHORTER);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(age_match_optmenu), menu);
@@ -616,36 +637,33 @@ static CondHBox *prefs_filter_edit_cond_hbox_create(void)
 	spin_btn_adj = gtk_adjustment_new(0, 0, 99999, 1, 10, 100);
 	spin_btn = gtk_spin_button_new(GTK_ADJUSTMENT(spin_btn_adj), 1, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), spin_btn, FALSE, FALSE, 0);
-	gtk_widget_set_usize(spin_btn, 64, -1);
+	gtk_widget_set_size_request(spin_btn, 64, -1);
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin_btn), TRUE);
 
 	label = gtk_label_new(_("KB"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
-	del_pixmap = stock_pixmap_widget(rule_edit_window.window,
-					 STOCK_PIXMAP_REMOVE);
-	gtk_widget_show(del_pixmap);
+	del_img = gtk_image_new_from_stock
+		(GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
+	gtk_widget_show(del_img);
 
 	del_btn = gtk_button_new();
-	gtk_container_add(GTK_CONTAINER(del_btn), del_pixmap);
+	gtk_container_add(GTK_CONTAINER(del_btn), del_img);
 	gtk_widget_show(del_btn);
 	gtk_box_pack_end(GTK_BOX(hbox), del_btn, FALSE, FALSE, 0);
 
-	add_pixmap = stock_pixmap_widget(rule_edit_window.window,
-					 STOCK_PIXMAP_ADD);
-	gtk_widget_show(add_pixmap);
+	add_img = gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
+	gtk_widget_show(add_img);
 
 	add_btn = gtk_button_new();
-	gtk_container_add(GTK_CONTAINER(add_btn), add_pixmap);
+	gtk_container_add(GTK_CONTAINER(add_btn), add_img);
 	gtk_widget_show(add_btn);
 	gtk_box_pack_end(GTK_BOX(hbox), add_btn, FALSE, FALSE, 0);
 
-	gtk_signal_connect(GTK_OBJECT(del_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_cond_del_cb),
-			   cond_hbox);
-	gtk_signal_connect(GTK_OBJECT(add_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_cond_add_cb),
-			   cond_hbox);
+	g_signal_connect(G_OBJECT(del_btn), "clicked",
+			 G_CALLBACK(prefs_filter_cond_del_cb), cond_hbox);
+	g_signal_connect(G_OBJECT(add_btn), "clicked",
+			 G_CALLBACK(prefs_filter_cond_add_cb), cond_hbox);
 
 	cond_hbox->hbox = hbox;
 	cond_hbox->cond_type_optmenu = cond_type_optmenu;
@@ -677,13 +695,13 @@ static ActionHBox *prefs_filter_edit_action_hbox_create(void)
 	GtkWidget *folder_entry;
 	GtkWidget *cmd_entry;
 	GtkWidget *address_entry;
-	GtkWidget *folder_pixmap;
+	GtkWidget *folder_img;
 	GtkWidget *folder_sel_btn;
 	GtkWidget *clabel_optmenu;
 	GtkWidget *del_btn;
 	GtkWidget *add_btn;
-	GtkWidget *del_pixmap;
-	GtkWidget *add_pixmap;
+	GtkWidget *del_img;
+	GtkWidget *add_img;
 
 	action_hbox = g_new0(ActionHBox, 1);
 
@@ -701,12 +719,13 @@ static ActionHBox *prefs_filter_edit_action_hbox_create(void)
 {									      \
 	MENUITEM_ADD(menu, menuitem, str, action);			      \
 	action_hbox->action_type_menu_items[action] = menuitem;		      \
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",		      \
-			   GTK_SIGNAL_FUNC(prefs_filter_action_activated_cb), \
-			   action_hbox);				      \
+	g_signal_connect(G_OBJECT(menuitem), "activate",		      \
+			 G_CALLBACK(prefs_filter_action_activated_cb),	      \
+			 action_hbox);					      \
 }
 
 	menu = gtk_menu_new();
+	gtk_widget_show(menu);
 	ACTION_MENUITEM_ADD(_("Move to"),               PF_ACTION_MOVE);
 	ACTION_MENUITEM_ADD(_("Copy to"),               PF_ACTION_COPY);
 	ACTION_MENUITEM_ADD(_("Don't receive"),         PF_ACTION_NOT_RECEIVE);
@@ -742,12 +761,12 @@ static ActionHBox *prefs_filter_edit_action_hbox_create(void)
 	gtk_widget_show(folder_entry);
 	gtk_box_pack_start(GTK_BOX(hbox), folder_entry, TRUE, TRUE, 0);
 
-	folder_pixmap = stock_pixmap_widget(rule_edit_window.window,
-					    STOCK_PIXMAP_DIR_OPEN);
-	gtk_widget_show(folder_pixmap);
+	folder_img = stock_pixmap_widget(rule_edit_window.window,
+					 STOCK_PIXMAP_DIR_OPEN);
+	gtk_widget_show(folder_img);
 
 	folder_sel_btn = gtk_button_new();
-	gtk_container_add(GTK_CONTAINER(folder_sel_btn), folder_pixmap);
+	gtk_container_add(GTK_CONTAINER(folder_sel_btn), folder_img);
 	gtk_widget_show(folder_sel_btn);
 	gtk_box_pack_start(GTK_BOX(hbox), folder_sel_btn, FALSE, FALSE, 0);
 
@@ -766,33 +785,30 @@ static ActionHBox *prefs_filter_edit_action_hbox_create(void)
 	menu = colorlabel_create_color_menu();
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(clabel_optmenu), menu);
 
-	del_pixmap = stock_pixmap_widget(rule_edit_window.window,
-					 STOCK_PIXMAP_REMOVE);
-	gtk_widget_show(del_pixmap);
+	del_img = gtk_image_new_from_stock
+		(GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
+	gtk_widget_show(del_img);
 
 	del_btn = gtk_button_new();
-	gtk_container_add(GTK_CONTAINER(del_btn), del_pixmap);
+	gtk_container_add(GTK_CONTAINER(del_btn), del_img);
 	gtk_widget_show(del_btn);
 	gtk_box_pack_end(GTK_BOX(hbox), del_btn, FALSE, FALSE, 0);
 
-	add_pixmap = stock_pixmap_widget(rule_edit_window.window,
-					 STOCK_PIXMAP_ADD);
-	gtk_widget_show(add_pixmap);
+	add_img = gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
+	gtk_widget_show(add_img);
 
 	add_btn = gtk_button_new();
-	gtk_container_add(GTK_CONTAINER(add_btn), add_pixmap);
+	gtk_container_add(GTK_CONTAINER(add_btn), add_img);
 	gtk_widget_show(add_btn);
 	gtk_box_pack_end(GTK_BOX(hbox), add_btn, FALSE, FALSE, 0);
 
-	gtk_signal_connect(GTK_OBJECT(folder_sel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_action_select_dest_cb),
-			   action_hbox);
-	gtk_signal_connect(GTK_OBJECT(del_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_action_del_cb),
-			   action_hbox);
-	gtk_signal_connect(GTK_OBJECT(add_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_action_add_cb),
-			   action_hbox);
+	g_signal_connect(G_OBJECT(folder_sel_btn), "clicked",
+			 G_CALLBACK(prefs_filter_action_select_dest_cb),
+			 action_hbox);
+	g_signal_connect(G_OBJECT(del_btn), "clicked",
+			 G_CALLBACK(prefs_filter_action_del_cb), action_hbox);
+	g_signal_connect(G_OBJECT(add_btn), "clicked",
+			 G_CALLBACK(prefs_filter_action_add_cb), action_hbox);
 
 	action_hbox->hbox = hbox;
 	action_hbox->action_type_optmenu = action_type_optmenu;
@@ -1028,6 +1044,7 @@ static void prefs_filter_edit_set_cond_hbox_widgets(CondHBox *hbox,
 		gtk_widget_hide(hbox->key_entry);
 		gtk_widget_show(hbox->spin_btn);
 		gtk_widget_show(hbox->label);
+		gtk_label_set_text(GTK_LABEL(hbox->label), _("KB"));
 		break;
 	case PF_COND_AGE:
 		gtk_widget_hide(hbox->match_type_optmenu);
@@ -1035,7 +1052,8 @@ static void prefs_filter_edit_set_cond_hbox_widgets(CondHBox *hbox,
 		gtk_widget_show(hbox->age_match_optmenu);
 		gtk_widget_hide(hbox->key_entry);
 		gtk_widget_show(hbox->spin_btn);
-		gtk_widget_hide(hbox->label);
+		gtk_widget_show(hbox->label);
+		gtk_label_set_text(GTK_LABEL(hbox->label), _("day(s)"));
 		break;
 	case PF_COND_ACCOUNT:
 		gtk_widget_hide(hbox->match_type_optmenu);
@@ -1212,7 +1230,8 @@ static ActionMenuType prefs_filter_edit_get_action_hbox_type(ActionHBox *hbox)
 	menuitem = gtk_menu_get_active
 		(GTK_MENU(gtk_option_menu_get_menu
 			(GTK_OPTION_MENU(hbox->action_type_optmenu))));
-	type = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+	type = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menuitem),
+						 MENU_VAL_ID));
 
 	return type;
 }
@@ -1360,7 +1379,7 @@ static void prefs_filter_edit_set_cond_header_menu(CondHBox *hbox)
 	while (child != NULL) {
 		GList *next = child->next;
 		menuitem = GTK_WIDGET(child->data);
-		if (!gtk_object_get_data(GTK_OBJECT(menuitem), "header_str"))
+		if (!g_object_get_data(G_OBJECT(menuitem), "header_str"))
 			break;
 		gtk_widget_destroy(menuitem);
 		child = next;
@@ -1373,13 +1392,13 @@ static void prefs_filter_edit_set_cond_header_menu(CondHBox *hbox)
 		menuitem = gtk_menu_item_new_with_label(header->name);
 		gtk_widget_show(menuitem);
 		gtk_menu_insert(GTK_MENU(menu), menuitem, pos);
-		gtk_object_set_user_data(GTK_OBJECT(menuitem),
-					 GINT_TO_POINTER(PF_COND_HEADER));
-		gtk_object_set_data(GTK_OBJECT(menuitem), "header_str",
-				    header->name);
-		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-				   GTK_SIGNAL_FUNC(prefs_filter_cond_activated_cb),
-				   hbox);
+		g_object_set_data(G_OBJECT(menuitem), MENU_VAL_ID,
+				  GINT_TO_POINTER(PF_COND_HEADER));
+		g_object_set_data(G_OBJECT(menuitem), "header_str",
+				  header->name);
+		g_signal_connect(G_OBJECT(menuitem), "activate",
+				 G_CALLBACK(prefs_filter_cond_activated_cb),
+				 hbox);
 	}
 
 	if (hbox->cur_type == PF_COND_HEADER)
@@ -1406,11 +1425,11 @@ static void prefs_filter_edit_activate_cond_header(const gchar *header)
 	for (cur = GTK_MENU_SHELL(menu)->children, index = 0;
 	     cur != NULL; cur = cur->next, index++) {
 		menuitem = GTK_WIDGET(cur->data);
-		menu_header = gtk_object_get_data(GTK_OBJECT(menuitem),
-						  "header_str");
+		menu_header = g_object_get_data(G_OBJECT(menuitem),
+						"header_str");
 		if (!menu_header)
 			break;
-		if (!g_strcasecmp(menu_header, header)) {
+		if (!g_ascii_strcasecmp(menu_header, header)) {
 			gtk_option_menu_set_history
 				(GTK_OPTION_MENU(hbox->cond_type_optmenu),
 				 index);
@@ -1427,19 +1446,19 @@ static gint edit_header_list_dialog_deleted(GtkWidget *widget,
 	return TRUE;
 }
 
-static gint edit_header_list_dialog_key_pressed(GtkWidget *widget,
-						GdkEventKey *event,
-						gpointer data)
+static gboolean edit_header_list_dialog_key_pressed(GtkWidget *widget,
+						    GdkEventKey *event,
+						    gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
 		edit_header_list_dialog.finished = TRUE;
-	return TRUE;
+	return FALSE;
 }
 
 static void edit_header_list_dialog_add(void)
 {
 	GtkCList *clist = GTK_CLIST(edit_header_list_dialog.clist);
-	gchar *text;
+	const gchar *text;
 	gchar *ctext[1];
 	gint row;
 	gchar *row_text;
@@ -1449,10 +1468,10 @@ static void edit_header_list_dialog_add(void)
 
 	for (row = 0; gtk_clist_get_text(clist, row, 0, &row_text) != 0;
 	     row++) {
-		if (g_strcasecmp(row_text, text) == 0) return;
+		if (g_ascii_strcasecmp(row_text, text) == 0) return;
 	}
 
-	ctext[0] = text;
+	ctext[0] = (gchar *)text;
 	gtk_clist_append(clist, ctext);
 }
 
@@ -1502,19 +1521,17 @@ static void prefs_filter_edit_edit_header_list_dialog_create(void)
 
 	gchar *title[1];
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width(GTK_CONTAINER(window), 8);
 	gtk_window_position(GTK_WINDOW(window),GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
 	gtk_window_set_title(GTK_WINDOW(window), _("Edit header list"));
 
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(edit_header_list_dialog_deleted),
-			   NULL);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(edit_header_list_dialog_key_pressed),
-			   NULL);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(edit_header_list_dialog_deleted), NULL);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(edit_header_list_dialog_key_pressed), NULL);
 
 	vbox = gtk_vbox_new(FALSE, 6);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
@@ -1526,7 +1543,7 @@ static void prefs_filter_edit_edit_header_list_dialog_create(void)
 	gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
 
 	scrwin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_widget_set_usize(scrwin, 120, 160);
+	gtk_widget_set_size_request(scrwin, 120, 160);
 	gtk_box_pack_start(GTK_BOX(vbox2), scrwin, TRUE, TRUE, 0);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwin),
 				       GTK_POLICY_AUTOMATIC,
@@ -1558,21 +1575,20 @@ static void prefs_filter_edit_edit_header_list_dialog_create(void)
 	del_btn = gtk_button_new_with_label(_(" Delete "));
 	gtk_box_pack_start(GTK_BOX(btn_vbox), del_btn, FALSE, FALSE, 0);
 
-	gtkut_button_set_create(&confirm_area, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
-	gtk_signal_connect(GTK_OBJECT(add_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_header_list_dialog_add), NULL);
-	gtk_signal_connect(GTK_OBJECT(del_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_header_list_dialog_delete),
-			   NULL);
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_header_list_dialog_ok), NULL);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_header_list_dialog_cancel),
-			   NULL);
+	g_signal_connect(G_OBJECT(add_btn), "clicked",
+			 G_CALLBACK(edit_header_list_dialog_add), NULL);
+	g_signal_connect(G_OBJECT(del_btn), "clicked",
+			 G_CALLBACK(edit_header_list_dialog_delete), NULL);
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(edit_header_list_dialog_ok), NULL);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(edit_header_list_dialog_cancel), NULL);
 
 	manage_window_set_transient(GTK_WINDOW(window));
 
@@ -1668,7 +1684,7 @@ static FilterRule *prefs_filter_edit_dialog_to_rule(void)
 		(GTK_MENU(gtk_option_menu_get_menu
 			(GTK_OPTION_MENU(rule_edit_window.bool_op_optmenu))));
 	bool_op = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(bool_op_menuitem)));
+		(g_object_get_data(G_OBJECT(bool_op_menuitem), MENU_VAL_ID));
 
 	for (cur = rule_edit_window.cond_hbox_list; cur != NULL;
 	     cur = cur->next) {
@@ -1689,12 +1705,11 @@ static FilterRule *prefs_filter_edit_dialog_to_rule(void)
 			(GTK_MENU(gtk_option_menu_get_menu
 				(GTK_OPTION_MENU(hbox->cond_type_optmenu))));
 		cond_menu_type = GPOINTER_TO_INT
-			(gtk_object_get_user_data
-				(GTK_OBJECT(cond_type_menuitem)));
+			(g_object_get_data
+				(G_OBJECT(cond_type_menuitem), MENU_VAL_ID));
 
-		match_menu_type = GPOINTER_TO_INT
-			(menu_get_option_menu_active_user_data
-				(GTK_OPTION_MENU(hbox->match_type_optmenu)));
+		match_menu_type = menu_get_option_menu_active_index
+			(GTK_OPTION_MENU(hbox->match_type_optmenu));
 
 		key_str = gtk_entry_get_text(GTK_ENTRY(hbox->key_entry));
 
@@ -1726,8 +1741,8 @@ static FilterRule *prefs_filter_edit_dialog_to_rule(void)
 
 		switch (cond_menu_type) {
 		case PF_COND_HEADER:
-			header_name = gtk_object_get_data
-				(GTK_OBJECT(cond_type_menuitem), "header_str");
+			header_name = g_object_get_data
+				(G_OBJECT(cond_type_menuitem), "header_str");
 			cond = filter_cond_new(FLT_COND_HEADER,
 					       match_type, match_flag,
 					       header_name, key_str);
@@ -1752,9 +1767,8 @@ static FilterRule *prefs_filter_edit_dialog_to_rule(void)
 				error_msg = _("Command is not specified.");
 			break;
 		case PF_COND_SIZE:
-			size_type = GPOINTER_TO_INT
-				(menu_get_option_menu_active_user_data
-				 (GTK_OPTION_MENU(hbox->size_match_optmenu)));
+			size_type = menu_get_option_menu_active_index
+				 (GTK_OPTION_MENU(hbox->size_match_optmenu));
 			match_flag = size_type == PF_SIZE_LARGER
 				? 0 : FLT_NOT_MATCH;
 			int_value = gtk_spin_button_get_value_as_int
@@ -1764,9 +1778,8 @@ static FilterRule *prefs_filter_edit_dialog_to_rule(void)
 					       itos(int_value));
 			break;
 		case PF_COND_AGE:
-			age_type = GPOINTER_TO_INT
-				(menu_get_option_menu_active_user_data
-				 (GTK_OPTION_MENU(hbox->age_match_optmenu)));
+			age_type = menu_get_option_menu_active_index
+				 (GTK_OPTION_MENU(hbox->age_match_optmenu));
 			match_flag = age_type == PF_AGE_LONGER
 				? 0 : FLT_NOT_MATCH;
 			int_value = gtk_spin_button_get_value_as_int
@@ -1904,12 +1917,13 @@ static gint prefs_filter_edit_deleted(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
-static gint prefs_filter_edit_key_pressed(GtkWidget *widget, GdkEventKey *event,
-					  gpointer data)
+static gboolean prefs_filter_edit_key_pressed(GtkWidget *widget,
+					      GdkEventKey *event,
+					      gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
 		prefs_filter_edit_cancel();
-	return TRUE;
+	return FALSE;
 }
 
 static void prefs_filter_edit_ok(void)
@@ -1934,7 +1948,8 @@ static void prefs_filter_cond_activated_cb(GtkWidget *widget, gpointer data)
 	CondHBox *hbox = (CondHBox *)data;
 	CondMenuType type;
 
-	type = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(widget)));
+	type = GPOINTER_TO_INT
+		(g_object_get_data(G_OBJECT(widget), MENU_VAL_ID));
 
 	if (type == PF_COND_EDIT_HEADER) {
 		prefs_filter_edit_edit_header_list();
@@ -1950,8 +1965,8 @@ static void prefs_filter_cond_activated_cb(GtkWidget *widget, gpointer data)
 			gchar *header_name;
 			gchar *header_field;
 
-			header_name = (gchar *)gtk_object_get_data
-				(GTK_OBJECT(widget), "header_str");
+			header_name = (gchar *)g_object_get_data
+				(G_OBJECT(widget), "header_str");
 			header_field = prefs_filter_get_msg_header_field
 				(header_name);
 			if (header_field)
@@ -1967,7 +1982,8 @@ static void prefs_filter_action_activated_cb(GtkWidget *widget, gpointer data)
 	ActionHBox *hbox = (ActionHBox *)data;
 	ActionMenuType type;
 
-	type = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(widget)));
+	type = GPOINTER_TO_INT
+		(g_object_get_data(G_OBJECT(widget), MENU_VAL_ID));
 	prefs_filter_edit_set_action_hbox_widgets(hbox, type);
 }
 
