@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,13 +22,13 @@
 #endif
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
 #include <sys/stat.h>
 
-#include "intl.h"
 #include "procheader.h"
 #include "procmsg.h"
 #include "codeconv.h"
@@ -37,7 +37,7 @@
 
 #define BUFFSIZE	8192
 
-gint procheader_get_one_field(gchar *buf, gint len, FILE *fp,
+gint procheader_get_one_field(gchar *buf, size_t len, FILE *fp,
 			      HeaderEntry hentry[])
 {
 	gint nexthead;
@@ -56,8 +56,8 @@ gint procheader_get_one_field(gchar *buf, gint len, FILE *fp,
 
 			for (hp = hentry, hnum = 0; hp->name != NULL;
 			     hp++, hnum++) {
-				if (!strncasecmp(hp->name, buf,
-						 strlen(hp->name)))
+				if (!g_ascii_strncasecmp(hp->name, buf,
+							 strlen(hp->name)))
 					break;
 			}
 		} while (hp->name == NULL);
@@ -147,7 +147,7 @@ gint procheader_get_one_field(gchar *buf, gint len, FILE *fp,
 	return hnum;
 }
 
-gchar *procheader_get_unfolded_line(gchar *buf, gint len, FILE *fp)
+gchar *procheader_get_unfolded_line(gchar *buf, size_t len, FILE *fp)
 {
 	gboolean folded = FALSE;
 	gint nexthead;
@@ -225,7 +225,7 @@ GSList *procheader_get_header_list_from_file(const gchar *file)
 
 GSList *procheader_get_header_list(FILE *fp)
 {
-	gchar buf[BUFFSIZE], tmp[BUFFSIZE];
+	gchar buf[BUFFSIZE];
 	gchar *p;
 	GSList *hlist = NULL;
 	Header *header;
@@ -240,8 +240,7 @@ GSList *procheader_get_header_list(FILE *fp)
 				header->name = g_strndup(buf, p - buf);
 				p++;
 				while (*p == ' ' || *p == '\t') p++;
-				conv_unmime_header(tmp, sizeof(tmp), p, NULL);
-				header->body = g_strdup(tmp);
+				header->body = conv_unmime_header(p, NULL);
 
 				hlist = g_slist_append(hlist, header);
 				break;
@@ -289,16 +288,16 @@ gint procheader_find_header_list(GSList *hlist, const gchar *header_name)
 
 	for (cur = hlist; cur != NULL; cur = cur->next, index++) {
 		header = (Header *)cur->data;
-		if (g_strcasecmp(header->name, header_name) == 0)
+		if (g_ascii_strcasecmp(header->name, header_name) == 0)
 			return index;
 	}
 
 	return -1;
 }
 
-GPtrArray *procheader_get_header_array(FILE *fp)
+GPtrArray *procheader_get_header_array(FILE *fp, const gchar *encoding)
 {
-	gchar buf[BUFFSIZE], tmp[BUFFSIZE];
+	gchar buf[BUFFSIZE];
 	gchar *p;
 	GPtrArray *headers;
 	Header *header;
@@ -315,8 +314,7 @@ GPtrArray *procheader_get_header_array(FILE *fp)
 				header->name = g_strndup(buf, p - buf);
 				p++;
 				while (*p == ' ' || *p == '\t') p++;
-				conv_unmime_header(tmp, sizeof(tmp), p, NULL);
-				header->body = g_strdup(tmp);
+				header->body = conv_unmime_header(p, encoding);
 
 				g_ptr_array_add(headers, header);
 				break;
@@ -327,9 +325,9 @@ GPtrArray *procheader_get_header_array(FILE *fp)
 	return headers;
 }
 
-GPtrArray *procheader_get_header_array_asis(FILE *fp)
+GPtrArray *procheader_get_header_array_asis(FILE *fp, const gchar *encoding)
 {
-	gchar buf[BUFFSIZE], tmp[BUFFSIZE];
+	gchar buf[BUFFSIZE];
 	gchar *p;
 	GPtrArray *headers;
 	Header *header;
@@ -345,8 +343,7 @@ GPtrArray *procheader_get_header_array_asis(FILE *fp)
 				header = g_new(Header, 1);
 				header->name = g_strndup(buf, p - buf);
 				p++;
-				conv_unmime_header(tmp, sizeof(tmp), p, NULL);
-				header->body = g_strdup(tmp);
+				header->body = conv_unmime_header(p, encoding);
 
 				g_ptr_array_add(headers, header);
 				break;
@@ -408,8 +405,8 @@ void procheader_get_header_fields(FILE *fp, HeaderEntry hentry[])
 
 		if (hp->body == NULL)
 			hp->body = g_strdup(p);
-		else if (!strcasecmp(hp->name, "To:") ||
-			 !strcasecmp(hp->name, "Cc:")) {
+		else if (!g_ascii_strcasecmp(hp->name, "To:") ||
+			 !g_ascii_strcasecmp(hp->name, "Cc:")) {
 			gchar *tp = hp->body;
 			hp->body = g_strconcat(tp, ", ", p, NULL);
 			g_free(tp);
@@ -505,12 +502,13 @@ MsgInfo *procheader_parse_stream(FILE *fp, MsgFlags flags, gboolean full)
 					    {NULL,		NULL, FALSE}};
 
 	MsgInfo *msginfo;
-	gchar buf[BUFFSIZE], tmp[BUFFSIZE];
-	gchar *reference = NULL;
+	gchar buf[BUFFSIZE];
 	gchar *p;
 	gchar *hp;
 	HeaderEntry *hentry;
 	gint hnum;
+	gchar *from = NULL, *to = NULL, *subject = NULL, *cc = NULL;
+	gchar *charset = NULL;
 
 	hentry = full ? hentry_full : hentry_short;
 
@@ -521,6 +519,7 @@ MsgInfo *procheader_parse_stream(FILE *fp, MsgFlags flags, gboolean full)
 
 	msginfo = g_new0(MsgInfo, 1);
 	msginfo->flags = flags;
+	msginfo->references = NULL;
 	msginfo->inreplyto = NULL;
 
 	while ((hnum = procheader_get_one_field(buf, sizeof(buf), fp, hentry))
@@ -536,20 +535,16 @@ MsgInfo *procheader_parse_stream(FILE *fp, MsgFlags flags, gboolean full)
 			msginfo->date = g_strdup(hp);
 			break;
 		case H_FROM:
-			if (msginfo->from) break;
-			conv_unmime_header(tmp, sizeof(tmp), hp, NULL);
-			msginfo->from = g_strdup(tmp);
-			msginfo->fromname = procheader_get_fromname(tmp);
+			if (from) break;
+			from = g_strdup(hp);
 			break;
 		case H_TO:
-			conv_unmime_header(tmp, sizeof(tmp), hp, NULL);
-			if (msginfo->to) {
-				p = msginfo->to;
-				msginfo->to =
-					g_strconcat(p, ", ", tmp, NULL);
+			if (to) {
+				p = to;
+				to = g_strconcat(p, ", ", hp, NULL);
 				g_free(p);
 			} else
-				msginfo->to = g_strdup(tmp);
+				to = g_strdup(hp);
 			break;
 		case H_NEWSGROUPS:
 			if (msginfo->newsgroups) {
@@ -562,8 +557,7 @@ MsgInfo *procheader_parse_stream(FILE *fp, MsgFlags flags, gboolean full)
 			break;
 		case H_SUBJECT:
 			if (msginfo->subject) break;
-			conv_unmime_header(tmp, sizeof(tmp), hp, NULL);
-			msginfo->subject = g_strdup(tmp);
+			subject = g_strdup(hp);
 			break;
 		case H_MSG_ID:
 			if (msginfo->msgid) break;
@@ -573,35 +567,41 @@ MsgInfo *procheader_parse_stream(FILE *fp, MsgFlags flags, gboolean full)
 			msginfo->msgid = g_strdup(hp);
 			break;
 		case H_REFERENCES:
+			msginfo->references =
+				references_list_prepend(msginfo->references,
+							hp);
+			break;
 		case H_IN_REPLY_TO:
-			if (!reference) {
-				eliminate_parenthesis(hp, '(', ')');
-				if ((p = strrchr(hp, '<')) != NULL &&
-				    strchr(p + 1, '>') != NULL) {
-					extract_parenthesis(p, '<', '>');
-					remove_space(p);
-					if (*p != '\0')
-						reference = g_strdup(p);
-				}
+			if (msginfo->inreplyto) break;
+
+			eliminate_parenthesis(hp, '(', ')');
+			if ((p = strrchr(hp, '<')) != NULL &&
+			    strchr(p + 1, '>') != NULL) {
+				extract_parenthesis(p, '<', '>');
+				remove_space(p);
+				if (*p != '\0')
+					msginfo->inreplyto = g_strdup(p);
 			}
 			break;
 		case H_CONTENT_TYPE:
-			if (!strncasecmp(hp, "multipart", 9))
+			if (!g_ascii_strncasecmp(hp, "multipart", 9)) {
 				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_MIME);
+			} else if (!charset) {
+				procmime_scan_content_type_str
+					(hp, NULL, &charset, NULL, NULL);
+			}
 			break;
 		case H_SEEN:
 			/* mnews Seen header */
 			MSG_UNSET_PERM_FLAGS(msginfo->flags, MSG_NEW|MSG_UNREAD);
 			break;
 		case H_CC:
-			conv_unmime_header(tmp, sizeof(tmp), hp, NULL);
-			if (msginfo->cc) {
-				p = msginfo->cc;
-				msginfo->cc =
-					g_strconcat(p, ", ", tmp, NULL);
+			if (cc) {
+				p = cc;
+				cc = g_strconcat(p, ", ", hp, NULL);
 				g_free(p);
 			} else
-				msginfo->cc = g_strdup(tmp);
+				cc = g_strdup(hp);
 			break;
 		case H_X_FACE:
 			if (msginfo->xface) break;
@@ -611,7 +611,30 @@ MsgInfo *procheader_parse_stream(FILE *fp, MsgFlags flags, gboolean full)
 			break;
 		}
 	}
-	msginfo->inreplyto = reference;
+
+	if (from) {
+		msginfo->from = conv_unmime_header(from, charset);
+		msginfo->fromname = procheader_get_fromname(msginfo->from);
+		g_free(from);
+	}
+	if (to) {
+		msginfo->to = conv_unmime_header(to, charset);
+		g_free(to);
+	}
+	if (subject) {
+		msginfo->subject = conv_unmime_header(subject, charset);
+		g_free(subject);
+	}
+	if (cc) {
+		msginfo->cc = conv_unmime_header(cc, charset);
+		g_free(cc);
+	}
+
+	if (!msginfo->inreplyto && msginfo->references)
+		msginfo->inreplyto =
+			g_strdup((gchar *)msginfo->references->data);
+
+	g_free(charset);
 
 	return msginfo;
 }
@@ -671,6 +694,10 @@ static gint procheader_scan_date_string(const gchar *str,
 			weekday, day, month, year, hh, mm, ss);
 	if (result == 7) return 0;
 
+	result = sscanf(str, "%d %9s %d %2d:%2d:%2d",
+			day, month, year, hh, mm, ss);
+	if (result == 6) return 0;
+
 	*ss = 0;
 	result = sscanf(str, "%10s %d %9s %d %2d:%2d %5s",
 			weekday, day, month, year, hh, mm, zone);
@@ -680,6 +707,15 @@ static gint procheader_scan_date_string(const gchar *str,
 			day, month, year, hh, mm, zone);
 	if (result == 6) return 0;
 
+	*zone = '\0';
+	result = sscanf(str, "%10s %d %9s %d %2d:%2d",
+			weekday, day, month, year, hh, mm);
+	if (result == 6) return 0;
+
+	result = sscanf(str, "%d %9s %d %2d:%2d",
+			day, month, year, hh, mm);
+	if (result == 5) return 0;
+
 	return -1;
 }
 
@@ -700,7 +736,6 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 
 	if (procheader_scan_date_string(src, weekday, &day, month, &year,
 					&hh, &mm, &ss, zone) < 0) {
-		g_warning("Invalid date: %s\n", src);
 		if (dest && len > 0)
 			strncpy2(dest, src, len);
 		return 0;
@@ -716,13 +751,11 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 
 	month[3] = '\0';
 	for (p = monthstr; *p != '\0'; p += 3) {
-		if (!strncasecmp(p, month, 3)) {
+		if (!g_ascii_strncasecmp(p, month, 3)) {
 			dmonth = (gint)(p - monthstr) / 3 + 1;
 			break;
 		}
 	}
-	if (*p == '\0')
-		g_warning("Invalid month: %s\n", month);
 
 	t.tm_sec = ss;
 	t.tm_min = mm;
@@ -749,11 +782,18 @@ void procheader_date_get_localtime(gchar *dest, gint len, const time_t timer)
 {
 	struct tm *lt;
 	gchar *default_format = "%y/%m/%d(%a) %H:%M";
+	gchar *tmp, *buf;
+
+	Xalloca(tmp, len + 1, dest[0] = '\0'; return;);
 
 	lt = localtime(&timer);
 
 	if (prefs_common.date_format)
-		strftime(dest, len, prefs_common.date_format, lt);
+		strftime(tmp, len, prefs_common.date_format, lt);
 	else
-		strftime(dest, len, default_format, lt);
+		strftime(tmp, len, default_format, lt);
+
+	buf = conv_localetodisp(tmp, NULL);
+	strncpy2(dest, buf, len);
+	g_free(buf);
 }
